# Test routines for the GGame tcl plugin
# by Gary Arndt

# Notes:
# - Although the some minor formatting issues and such were changed recently, most of 
# this was written many years ago as nothing more than a test, so there may still be 
# things that need to be cleaned up.
# - This routine depends only on core tcl, the GGame tcl plugin, and the related 
# custom init script.  It is free from any other dependencies.
# - Tcl is NOT required to script GGame, but allows for the additional flexibility of 
# a full scripting language.  These routines are intended to demonstrate that 
# capability.
# - "gpi" refers to "GGame plugin" and should be used as a prefix to all data files 
# stored in the work dir, etc..
# - "tpi" refers to "tcl plugin" and is typically used when it is necessary to 
# clarify that a given identifier or name is based on such.  The TEA commands from 
# the plugin currently use this prefix, but they could have also used gpi.

#######################################################################################
# Echo loop as a basic test, particularly of text IO

proc EchoLoop {} {

	# Intro text
	msg "*** Echo input test loop ***"
	msg "(<Return> to exit)"

	# Input loop
	while {1} {

		# Prompt for input
		msg -nnl "> "
		set sInput [input]

		# Check for exit
		if {$sInput == ""} break;

		# Echo input
		msg "Input: $sInput"
	}
}


#######################################################################################
# Loop for testing SCMD commands

proc SCMDLoop {} {

	msg "*** SCMD loop ***"
	msg "(<Return> to exit)"

	# Command loop
	while {1} {

		# Prompt for input
		msg -nnl "command? "
		set sInput [input]

		# Check for exit
		if {$sInput == ""} break;

		# Run command
		catch {tpiSCMD $sInput}
	}
}


#######################################################################################
# Save the values of a given array into the given data file

# - This is a very useful utility function that goes beyond just being a test

proc SaveDataFile {sFile ppData} {

	upvar $ppData pData

	# Open file
	set hFile [open $sFile w]

	# Write data
	foreach sField [array names pData] {

		# Debug
#		msg "DBG: \"$sField\" = \"$pData($sField)\""

		# Write data to file
		puts $hFile "$sField = $pData($sField)"
	}

	# Close file
	close $hFile
}


#######################################################################################
# Load a given data file into the given array

# - This is a very useful utility function that goes beyond just being a test

proc LoadDataFile {sFile ppData} {

	upvar $ppData pData

	# Open file
	set hFile [open $sFile r]

	# Read data
	while {![eof $hFile]} {

		# Get line
		set sLine [gets $hFile]

		# Debug
		msg "DBG: \"sLine\" = \"$sLine\""

		# Parse data into array
		if {$sLine != ""} {
			set sField [lindex $sLine 0]
			set pData($sField) [lindex $sLine 2]
		}
	}

	close $hFile
}


#######################################################################################
# Complex conversation example

proc Aticus {} {

	# Determine save file
	set sSaveFile "[tpiCMD GetWorkDir]/gpi-Aticus.dat"

	# Load save values if the save file exists
	if {[file exists $sSaveFile]} {
		LoadDataFile $sSaveFile pSaveValues

	# Otherwise initialize save values
	} else {
		set pSaveValues(HeldGold) 0
	}

	# Only save the save values if they change
	set bSVChanged 0

	# Intro text
	msg "Well met.  I am known as Aticus."

	# Conversation loop
	while {1} {

		# Prompt for input
		msg -nnl "> "
		set sInput [input]

		# Check for exit
		if {$sInput == ""} break

		# Resolve the keyword
		switch -glob -- [string tolower $sInput] {

		name {
			msg "AT-I-CUS! Remember?"
		}
		job {
			msg "I am a wizard!  I can do many things."
		}
		wizard {
			msg "Yes, hex-a-poo and all that"
		}
		things {
			msg "Fine, I'll show you.  And don't tell Zaranth I did this, this is sort of his gig.  I could turn that table over there into the trees from which it was made!  Or I could summon a jester, although I hate that one.  Too many jesters in this world already.  I can make it dark around here, or light again afterwards.  I'm as much a humanitarian as Mr. Avatar over there, too: I can heal you if you are hurt, or give you food if you are hungry.  I even know something about your quest and could give you a hint about it."
		}
		table {
			msg "ALACAZAM!"
			tpiSCMD FX 1 104
			tpiSCMD AlterTiles 5 5 18 11 18
			msg "I can also turn it back, by the way."
		}
		back {
			msg "ALACAZAM!"
			tpiSCMD FX 1 104
			# Left:
			tpiSCMD AlterTiles 11 5 18
			# Middle:
			tpiSCMD AlterTiles 10 6 18 10 18
			# Right:
			tpiSCMD AlterTiles 12 11 18
			msg "There, good as new.  Ahem.  Well, try not to lean on it, though."
		}
		jester {
			msg "Ugh, really?  There's so dull they have to do their own laughing.  I'm MUCH funnier than they are.  Ok, well here you are then.  ALACAZAM!"
			tpiSCMD FX 1 104
			tpiSCMD MakeMapObj Creature 21 8 17 6
			msg "Unfortunately, I can't get rid of them again like turning the table back.  Let's not make anymore, ok?"
		}
		hint {
			if [tpiCMD GetFlag 1 1] {
				msg "Hey, you have the ring, you don't need my help."
			} else {
				msg "Ok, let me make sure I've got this right: \"Find the ring, earn the thing!\"  No, wait, that's not right ... oh, yeah, ok: \"Find the father, earn the ring!\""
			}
		}
		dark {
			if [tpiCMD GetDarkness] {
				msg "Let's not, it's pretty dark already."
			} else {
				msg "ALACAZAM!"
				tpiSCMD FX 1 104
				tpiSCMD SetDarkness DARK
				msg "WOW.  This is REALLY dark.  Um, how about we turn it light again?  Otherwise we're all going to need torches around here."
			}
		}
		light {
			if {![tpiCMD GetDarkness]} {
				msg "Sorry, I can't make it much sunnier than this without some serious sunblock.  Ok, ok, I know, but hey, if there are rockets, why can't there at least be sunblock?  Sorry, that's \"burnblock\", invented by the gnomes!  Yeah, that's it!"
			} else {
				msg "ALACAZAM!"
				tpiSCMD FX 1 104
				tpiSCMD SetDarkness LIGHT
				msg "That's much better!"
			}
		}
		heal {
			msg "ALACAZAM!"
			tpiSCMD FX 1 104
			tpiSCMD Heal 0d0+10
			msg "If you still need more, just ask."
		}
		food {
			msg "Of course, you look like you haven't had anything in a while.  Here you go."
			tpiSCMD GiveItem 2 100
		}
		give {
			msg "No thanks, that's very kind of you, but I don't want your gold.  But I CAN hold onto it for a while for you and give it back when you are ready.  It must get pretty heavy carrying all those coins around all the time.  How much do you want me to hang onto for you?"

			# Get amount
			msg -nnl "> "
			set nAmount [input]
			if {$nAmount > 0} {

				# Verify there is enough gold in inventory
				set nInvAmount [tpiCMD GetItemCount 1]
				if {$nAmount > $nInvAmount} {
					set nAmount $nInvAmount
				}

				# Take gold from inventory
				tpiSCMD TakeItem 1 $nAmount

				# Add to total held
				incr pSaveValues(HeldGold) $nAmount
				set bSVChanged 1

				msg "When you want these back, just ask me about your gold."

			} else {
				msg "None?  Ok, no problem."
			}
		}
		gold {
			set nHeldGold $pSaveValues(HeldGold)

			# Holding gold
			if {$nHeldGold > 0} {
				msg "I am holding $nHeldGold gp of your gold.  If you want some of it back, how much?"

				# Get amount
				msg -nnl "> "
				set nAmount [input]
				if {$nAmount > 0} {

					# Make sure it isn't more than is held
					if {$nAmount > $nHeldGold} {
						# If it is, just adjust the value down to the total held
						set nAmount $nHeldGold
					}

					# Subtract from total
					incr pSaveValues(HeldGold) -$nAmount
					set bSVChanged 1

					# Put gold in inventory
					tpiSCMD GiveItem 1 $nAmount

					msg "Ok, here's your $nAmount gold back"

				} else {
					msg "None? Ok, no problem.  I'll keep holding onto these until you need them."
				}

			# Not holding any gold
			} else {
				msg "Sorry, I don't carry a lot of gold myself, nor am I holding any of your gold for you right now."
			}
		}
		win {
			# The following is for the sake of humor; I apologize if anyone is offended (I wrote it a long time ago and don't really want to remove it because it made me laugh when I read it recently)
			msg "Isn't this kind of cheating?  You can't REALLY win until you do something about Minax.  She's a really nasty wizardess, needs a good spanking ... uh, I mean, something must be done about her!  But what do I know?"
			msg "Win-a-poo-win-on-you!"
			tpiSCMD FX 1 104
			tpiSCMD Win
			break
		}
		zaranth {
			msg "[He rolls his eyes].  He's such a stiff.  But he is the Avatar, and actually a pretty decent guy.  I just have to give him a hard time.  Thinks he can be an expert paladin AND a wizard.  Going to put us robed guys with our sticks out of business!"
		}
		bye {
			msg "Catch you lat ... uh, I mean, \"fare thee well\""
			break
		}
		default {
			msg "I cannot help thee with that even if it IS a synonym for something I CAN help thee with.  Sorry, I'm a bit picky that way.  Wizard thing.  Well, actually, pretty much everyone around here does it.  That is, everyone who can be bothered to say more than one thing over and over.  Sheesh."
		}

		}
	}

	# Save pSaveValues if they changed
	if {$bSVChanged} {
		SaveDataFile $sSaveFile pSaveValues
	}
}


#######################################################################################
# Complex item example

# UNDER CONSTRUCTION!

# TODO:
# - Implement a magical location that will recharge this wand
# - Implement a magical ship

proc ShipWand {} {

	# Init
	set sFile "[tpiCMD GetWorkDir]/gpi-ShipWand.dat"
	set sDataChange 0

	# Set values
	if {[file exists $sFile]} {
		msg "TODO: Read data file values"
	} else {
		set pData(nCharges) 100
		set pData(bShipInUse) 0
	}

	# Intro text
	msg "You hold the wand.  What do you do?"

	# Interaction loop
	while {1} {

		# Prompt for input
		msg -nnl "> "
		set sInput [input]

		# Check for exit
		if {$sInput == ""} break;

		# Resolve the keyword
		switch -glob -- [string tolower $sInput] {

		look {
			msg "The wand has magical runes all over it, but you cannot read them."
		}
		use {
			msg "Nothing happens.  Perhaps you need the command word."
		}
		test1 -
		azxiffm {
			msg "You speak a command word and a ship appears!"
			msg {[Effect TBI]}

			# Use charges
			incr pData(nCharges) -1

			# Track that the ship is currently on the map
			set pData(bShipInUse) 1

			# Data changed
			set sDataChange 1
		}
		test2 -
		amefai {
			msg "You speak the command word and the ground turns to sea!"
			msg {[Effect TBI]}

			# Use charges
			incr pData(nCharges) -2

			# Data changed
			set sDataChange 1
		}
		nothing {
			msg "You put the wand away."
			break
		}
		default {
			msg "That does nothing."
		}

		}
	}

	# Save values
	if {$sDataChange} {
		SaveDataFile $sFile pData
	}
}
